start_obs <- function(var, sys, frequency, year_in, year_fi, mm, lat_min, lat_max, lon_min, lon_max){
  library(startR)
  
  sdates <- paste0(year_in:year_fi, mm)
  
  if(sys == 'era5'){
    cds_path <- TRUE # it's the path in /esarchive with updated data from CDS for ERA5; era5land doesn't have this path
  }else{
    cds_path <- FALSE
  }
  
  fh <- if(frequency == 'daily'){'f1h'}else{if(var == 'tasmax' | var == 'tasmin'){'f24h'}else{'f1h'}} 
  # in the case of monthly_mean tas_max(min), 
  # f24h is the monthly mean of the daily max (min), 
  # while f1h is the monthly mean of the hourly max (min)
  # so in the case of monthly tas_max and tas_min, we want f24h
  # in any other case (prlr or daily data) we want f1h
  
  ref_path <- paste0('/esarchive/recon/ecmwf/', sys, '/', frequency, '_mean/')
  repos_obs <- paste0(ref_path, '$var$_', fh, if(cds_path == TRUE){'-r1440x721cds'}, '/$var$_$sdate$.nc') 
  # ref.path <- paste0(ref_path, paste0(var, '_', fh), if(cds_path == TRUE){'-r1440x721cds'}, '/')
  obs <- Start(dat = repos_obs,
               var = var,
               sdate = sdates,
               time = indices(1),
               latitude = values(list(lat_min, lat_max)),
               latitude_reorder = Sort(decreasing = TRUE),
               longitude = values(list(lon_min, lon_max)),
               longitude_reorder = CircularSort(0, 360),
               synonims = list(latitude=c('lat','latitude'),
                               longitude=c('lon','longitude')),
               #transform = CDORemapper,
               #transform_extra_cells = 2,
               #transform_params = list(grid = 'r256x128',
               #                           method = 'conservative',
               #                           crop = c(lon_min,lon_max, lat_min,lat_max)),
               #transform_vars = c('latitude', 'longitude'),
               return_vars = list(latitude = 'dat',
                                  longitude = 'dat',
                                  time = NULL),
               retrieve = TRUE)
  lats.ref <- attr(obs, 'Variables')$dat1$latitude
  lons.ref <- attr(obs, 'Variables')$dat1$longitude
  source_files <- attr(obs, 'Files')
  
  # dates as list for s2dv_cube object:        
  dates_for_s2dv <- list(start = array(sdates[1:length(sdates)], dim=c(1,1,length(sdates)), dimnames = list('sdate', 'time')), 
                         end = array(rev(rev(sdates)[1:length(sdates)]), dim=c(1,1,length(sdates)), dimnames = list('sdate', 'time')))
  names(dim(obs)) <- replace(names(dim(obs)), which(names(dim(obs)) == 'latitude'), 'lat')
  names(dim(obs)) <- replace(names(dim(obs)), which(names(dim(obs)) == 'longitude'), 'lon')
  names(dim(lats.ref)) <- 'lat'
  names(dim(lons.ref)) <- 'lon'
  library(CSTools) # function s2dv_cube is from CSTools
  loaded_data_ref_s2dv <- s2dv_cube(data = obs, lat = lats.ref, lon = lons.ref, Variable = list(varName = var, level = NULL), when = Sys.time(), source_files = source_files, Dates = dates_for_s2dv)
  attr(loaded_data_ref_s2dv$lon,"first_lon") <- lon_min
  attr(loaded_data_ref_s2dv$lon,"last_lon") <- lon_max
  attr(loaded_data_ref_s2dv$lat,"first_lat") <- lat_min
  attr(loaded_data_ref_s2dv$lat,"last_lat") <- lat_max
  
  
  return(loaded_data_ref_s2dv)
}

load_monthly_data_all_variables_and_months <- function(year_in, year_fi, lat_min, lat_max, lon_min, lon_max){
  library(abind)
  variables <- c('tasmax', 'tasmin', 'prlr')
  months <- c('01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12')
  for(var in variables){
    for(mm in months){
      data <- start_obs(
        var = var, # tas, tasmax, tasmin, prlr, ...
        sys = 'era5land', # era5, era5land
        frequency = 'monthly', # monthly, daily
        year_in = year_in, 
        year_fi = year_fi, 
        mm = mm, 
        lat_min = lat_min, 
        lat_max = lat_max, 
        lon_min = lon_min, 
        lon_max = lon_max)
      
      if(mm == months[1]){
        loaded_data <- data
        dimnames <- names(dim(loaded_data$data))
        loaded_data$data <- abind(data$data, along = 4) # dim 4 is 'time'
        names(dim(loaded_data$data)) <- dimnames
      } else {
        loaded_data$data <- abind(loaded_data$data, data$data, along = 4)
        names(dim(loaded_data$data)) <- dimnames
      }
    }
    if(var == 'tasmax'){
      loaded_tasmax <- loaded_data
    }
    if(var == 'tasmin'){
      loaded_tasmin <- loaded_data
    }
    if(var == 'prlr'){
      loaded_prlr <- loaded_data
    }
  }
  return(list(tasmax_obs = loaded_tasmax, tasmin_obs = loaded_tasmin, prlr_obs = loaded_prlr))
}
